local insetValue = 2;

CUI_FRAME_BACKGROUND = "Interface\\AddOns\\CleanUI\\skins\\frame\\frame-background";
CUI_FRAME_GRADIENT_BACKGROUND = "Interface\\AddOns\\CleanUI\\skins\\frame\\gradient-background";

CUI_FRAME_BORDER = "Interface\\AddOns\\CleanUI\\skins\\frame\\frame-border";
CUI_FRAME_BORDER_WIDE = "Interface\\AddOns\\CleanUI\\skins\\frame\\frame-border-wide";

function CleanUI_RemoveBackdrop(frame)
    frame:SetBackdrop(nil);
    local backdropFrame = _G[frame:GetName().."BackdropFrame"];
    if (backdropFrame) then
        backdropFrame:SetBackdrop(nil);
    end
end

function CleanUI_SetGradientBackdrop(frame, top, left, bottom, right)
    local parent = frame;

    if (top or left or bottom or right) then
        local backdropFrame = _G[frame:GetName().."BackdropFrame"];
        if (not backdropFrame) then
            backdropFrame = CreateFrame("Frame", frame:GetName().."BackdropFrame", frame);
            backdropFrame:SetParent(frame);
            backdropFrame:SetFrameStrata("BACKGROUND");
        end

        backdropFrame:ClearAllPoints();
        backdropFrame:SetPoint("TOPLEFT", frame, "TOPLEFT", (left or 0), (top or 0));
        backdropFrame:SetPoint("BOTTOMRIGHT", frame, "BOTTOMRIGHT", (right or 0), (bottom or 0));

        parent = backdropFrame;
    end

    parent:SetBackdrop( {
        bgFile = CUI_FRAME_GRADIENT_BACKGROUND,
        tile = false,
        insets = { left = insetValue, right = insetValue, top = insetValue, bottom = insetValue }
    });
end

function CleanUI_SetBackdrop(frame, addBackground, borderType, top, left, bottom, right)
    local backdropFrame = _G[frame:GetName().."BackdropFrame"];
    if (not backdropFrame) then
        backdropFrame = CreateFrame("Frame", frame:GetName().."BackdropFrame", frame, BackdropTemplateMixin and "BackdropTemplate");
        backdropFrame:SetParent(frame);
        backdropFrame:SetFrameStrata("BACKGROUND");
    end

    backdropFrame:ClearAllPoints();
    backdropFrame:SetPoint("TOPLEFT", frame, "TOPLEFT", (left or 0), (top or 0));
    backdropFrame:SetPoint("BOTTOMRIGHT", frame, "BOTTOMRIGHT", (right or 0), (bottom or 0));

    local parent = backdropFrame;

    local border = CUI_FRAME_BORDER;

    if (borderType and borderType == "wide") then
        border = CUI_FRAME_BORDER_WIDE;
    end


    if (addBackground == false) then
        parent:SetBackdrop( {
            edgeFile = border, tile = true, tileSize = 64, edgeSize = 16,
            insets = { left = insetValue, right = insetValue, top = insetValue, bottom = insetValue }
        });
    else
        parent:SetBackdrop( {
            bgFile = CUI_FRAME_BACKGROUND,
            edgeFile = border, tile = true, tileSize = 64, edgeSize = 16,
            insets = { left = insetValue, right = insetValue, top = insetValue, bottom = insetValue }
        });
    end

    parent:SetBackdropColor(0.0, 0.0, 0.0, 1.0);
    --parent:SetBackdropBorderColor(0.0, 0.0, 0.0, 1.0);
end

function CleanUI_SetArtBackdrop(frame, top, left, bottom, right)
    local texture = "Interface\\PLAYERACTIONBARALT\\GENERICWOW";

    top = top or 0;
    left = left or 0;
    bottom = bottom or 0;
    right = right or 0;

    local cuiArtFrame = _G[frame:GetName().."ArtBackdrop"];
    if (not cuiArtFrame) then
        cuiArtFrame = CreateFrame("Frame", frame:GetName().."ArtBackdrop", frame, BackdropTemplateMixin and "BackdropTemplate");
    end

    cuiArtFrame:SetParent(frame);
    cuiArtFrame:SetPoint("TOPLEFT", frame, "TOPLEFT", left, top);
    cuiArtFrame:SetPoint("BOTTOMRIGHT", frame, "BOTTOMRIGHT", right, bottom);
    cuiArtFrame:SetFrameStrata("BACKGROUND");
    
    cuiArtFrame:SetBackdrop( {
            bgFile = nil,
            edgeFile = "Interface\\AddOns\\CleanUI\\skins\\minimap\\minimap-border",
            tile = true, tileSize = 32, edgeSize = 8.5,
            insets = { left = 5, right = 5, top = 5, bottom = 5 }
        });
    
    if (not cuiArtFrame.background) then
        cuiArtFrame.background = cuiArtFrame:CreateTexture("ARTWORK");
        cuiArtFrame.background:SetPoint("TOPLEFT", cuiArtFrame, "TOPLEFT", 8, -3);
        cuiArtFrame.background:SetPoint("BOTTOMRIGHT", cuiArtFrame, "BOTTOMRIGHT", -8, 3);
        cuiArtFrame.background:SetTexture(texture);
        cuiArtFrame.background:SetTexCoord(0.0, 1.0, 0.1757, 0.3555);
    end
    
    if (not cuiArtFrame.topleftborder) then
        cuiArtFrame.topleftborder = cuiArtFrame:CreateTexture("HIGHLIGHT");
        cuiArtFrame.topleftborder:SetParent(frame);
        cuiArtFrame.topleftborder:SetPoint("TOPLEFT", cuiArtFrame, "TOPLEFT", 0, 1);
        cuiArtFrame.topleftborder:SetPoint("BOTTOMRIGHT", cuiArtFrame, "TOPLEFT", 18, -22);
        cuiArtFrame.topleftborder:SetTexture(texture);
        cuiArtFrame.topleftborder:SetTexCoord(0.2402, 0.3027, 0.7910, 0.8710);
    end
    
    if (not cuiArtFrame.toprightborder) then
        cuiArtFrame.toprightborder = cuiArtFrame:CreateTexture("HIGHLIGHT");
        cuiArtFrame.toprightborder:SetParent(frame);
        cuiArtFrame.toprightborder:SetPoint("TOPLEFT", cuiArtFrame, "TOPRIGHT", -18, 1);
        cuiArtFrame.toprightborder:SetPoint("BOTTOMRIGHT", cuiArtFrame, "TOPRIGHT", 0, -22);
        cuiArtFrame.toprightborder:SetTexture(texture);
        cuiArtFrame.toprightborder:SetTexCoord(0.3027, 0.2402, 0.7910, 0.8710);
    end
    
    if (not cuiArtFrame.bottomleftborder) then
        cuiArtFrame.bottomleftborder = cuiArtFrame:CreateTexture("HIGHLIGHT");
        cuiArtFrame.bottomleftborder:SetParent(frame);
        cuiArtFrame.bottomleftborder:SetPoint("TOPLEFT", cuiArtFrame, "BOTTOMLEFT", 0, 22);
        cuiArtFrame.bottomleftborder:SetPoint("BOTTOMRIGHT", cuiArtFrame, "BOTTOMLEFT", 18, -1);
        cuiArtFrame.bottomleftborder:SetTexture(texture);
        cuiArtFrame.bottomleftborder:SetTexCoord(0.2402, 0.3027, 0.8710, 0.7910);
    end
    
    if (not cuiArtFrame.bottomrightborder) then
        cuiArtFrame.bottomrightborder = cuiArtFrame:CreateTexture("HIGHLIGHT");
        cuiArtFrame.bottomrightborder:SetParent(frame);
        cuiArtFrame.bottomrightborder:SetPoint("TOPLEFT", cuiArtFrame, "BOTTOMRIGHT", -18, 22);
        cuiArtFrame.bottomrightborder:SetPoint("BOTTOMRIGHT", cuiArtFrame, "BOTTOMRIGHT", 0, -1);
        cuiArtFrame.bottomrightborder:SetTexture(texture);
        cuiArtFrame.bottomrightborder:SetTexCoord(0.3027, 0.2402, 0.8710, 0.7910);
    end

    return cuiArtFrame;
end

function CleanUI_UpdateAccountData()
    local guid = UnitGUID("player");
    
    local name = UnitName("player");
    
    CleanUIAccountData[guid] = CleanUIAccountData[guid] or {};
    
    CleanUIAccountData[guid].name = UnitName("player");
    CleanUIAccountData[guid].realm = GetRealmName();
    CleanUIAccountData[guid].class = UnitClass("player");
    CleanUIAccountData[guid].race = UnitRace("player");
    CleanUIAccountData[guid].faction = UnitFactionGroup("player");
    CleanUIAccountData[guid].level = UnitLevel("player");
end

function CleanUI_ResetMoney()
    CleanUICurrency["money"] = {};
end

function CleanUI_UpdateMoney()
    CleanUICurrency["money"] = CleanUICurrency["money"] or {};
    
    local money = CleanUICurrency["money"];
    
    money[UnitGUID("player")] = GetMoney();
end

