local oldname = nil;

local equipmentSetItems = nil;

function CleanUI_InitEquipmentSets()
    CleanUI_EquipmentSetsHookEvents();
    CleanUI_RegisterEquipmentEvents();
    
    CleanUI_UpdateEquipmentSetItems();
end

function CleanUI_RegisterEquipmentEvents()
    local frame = CreateFrame("Frame");

    frame:RegisterEvent("EQUIPMENT_SETS_CHANGED");
    frame:RegisterEvent("PLAYER_SPECIALIZATION_CHANGED");

    frame:HookScript("OnEvent", CleanUI_EquipmentOnEvent);
end

function CleanUI_EquipmentOnEvent(self, event, ...)

    if (event == "EQUIPMENT_SETS_CHANGED") then
        CleanUI_UpdateEquipmentSetItems();
    end

    if (event == "PLAYER_SPECIALIZATION_CHANGED") then
        CleanUI_UpdateEquipmentSetForSpec();
    end
end 

function CleanUI_UpdateEquipmentSetItems()
    equipmentSetItems = {};
    
    local name, icon, set, itemArray;
    local itemName, itemLink;

    local setIDs = C_EquipmentSet.GetEquipmentSetIDs();
    for i = 1, #setIDs do
        itemArray = C_EquipmentSet.GetItemIDs(setIDs[i]);
        for i=1, 19 do
            if itemArray[i] then
                itemName, itemLink = GetItemInfo(itemArray[i]);
                tinsert(equipmentSetItems, itemLink);
            end
        end
    end
end

function CleanUI_UpdateEquipmentSetForSpec()
    local currentSpec = GetSpecialization();
    local id, name = GetSpecializationInfo(currentSpec, nil, nil, nil, UnitSex("player"));
    
    if (name) then
        CleanUI_SelectEquipmentSet(name);
    end
end 

function CleanUI_IsEquipmentSetItem(itemLink)
    if (not equipmentSetItems) then
        CleanUI_UpdateEquipmentSetItems();
    end

    for i,foundLink in pairs(equipmentSetItems) do
        local id1 = foundLink:match("item:(%d+)");
        local id2 = itemLink:match("item:(%d+)");
        if (id1 == id2) then
            return true;
        end
    end

    return false;
end

function CleanUI_EquipmentSetsHookEvents()
    hooksecurefunc("UnitPopup_ShowMenu", CleanUI_UnitPopup_AppendEquipmentSets);
    hooksecurefunc("UnitPopup_OnClick", CleanUI_UnitPopup_EquipmentSets_OnClick);
end

function CleanUI_UnitPopup_EquipmentSets_OnClick(self)
	local button = self.value;

    if (string.find(button, "CUI_EQUIPMENT_SETS") == 1) then
        UnitPopupButtons[button].func()
    end
end

function CleanUI_UnitPopup_AppendEquipmentSets(dropdownMenu, which, unit, name, userData)
    if (unit ~= "player" or UIDROPDOWNMENU_MENU_LEVEL > 1) then
        return;
    end

    if (C_EquipmentSet.GetNumEquipmentSets() > 0) then

        local menu = {};

        CleanUI_DetectEqippedSet();

        local equipmentSetIDs = C_EquipmentSet.GetEquipmentSetIDs();
        for i, setId in ipairs(equipmentSetIDs) do
            local name = C_EquipmentSet.GetEquipmentSetInfo(setId);
            local func = function(frame) CleanUI_SelectEquipmentSet(name) end;
            local checked = CleanUIUserData.EQUIPPED_SETNAME == name

            UnitPopupButtons["CUI_EQUIPMENT_SETS_"..name] = {text=name, value="CUI_123", func=func, checked=checked};
            table.insert(menu, "CUI_EQUIPMENT_SETS_"..name);
        end

        UnitPopupMenus["CUI_EQUIPMENT_SETS"] = menu;

        UIDropDownMenu_AddSeparator();

        local eq_header = UIDropDownMenu_CreateInfo();
        eq_header.text = "EquipmentSets";
        eq_header.value = "CUI_EQUIPMENT_SETS";
        eq_header.isNotRadio = true;
        eq_header.notCheckable = 1;
        eq_header.hasArrow = true;
        eq_header.menuTable = menu;
        UIDropDownMenu_AddButton(eq_header);
    end
end

function CleanUI_DetectEqippedSet()
    
    local equipmentSetIDs = C_EquipmentSet.GetEquipmentSetIDs();

    for i, setId in ipairs(equipmentSetIDs) do        
        local array = C_EquipmentSet.GetItemLocations(setId);
        local name = C_EquipmentSet.GetEquipmentSetInfo(setId);

        local checksum = 0;
        local location, ltest;

        for index=1, #array do
            location = array[index];

            -- check if location is an id
            if (location and tonumber(""..location) ~= nil) then
                --it's a number

                local player, bank, bags, voidStorage, slot, bag = EquipmentManager_UnpackLocation(location);

                if (player and not (bank or bags or bag)) then
                    --DEFAULT_CHAT_FRAME:AddMessage("equipped", 1.0, 1.0, 1.0, 1, 10);
                else
                    if (slot and slot > 1) then
                        checksum = checksum - 1;
                    end
                end
            end;
        end

        if (checksum == 0) then
            CleanUIUserData.EQUIPPED_SETNAME = name;
            return;
        end
    end

    CleanUIUserData.EQUIPPED_SETNAME = nil;
end

function CleanUI_SelectEquipmentSet(name)
    local equipmentSetIDs = C_EquipmentSet.GetEquipmentSetIDs();
    
    local foundName;
    for i, setId in ipairs(equipmentSetIDs) do
        foundName = C_EquipmentSet.GetEquipmentSetInfo(setId);
        
        if (foundName == name) then
            C_EquipmentSet.UseEquipmentSet(setId);
            PlaySound(SOUNDKIT.IG_MAINMENU_OPEN);
        end
    end
end
