local oldname = nil;

local equipmentSetItems = nil;

function CleanUI_InitGameMenu()
    CleanUI_GameMenuHookEvents();
    CleanUI_RegisterGameMenu();
    
    CleanUI_UpdateGameMenuBlackout();
    CleanUI_UpdateGameMenu();
end

function CleanUI_RegisterGameMenu()
    local frame = CreateFrame("Frame");

    --frame:RegisterEvent("EQUIPMENT_SETS_CHANGED");

    frame:HookScript("OnEvent", CleanUI_GameMenuOnEvent); 
end

function CleanUI_GameMenuOnEvent(self, event, ...)

    --if (event == "EQUIPMENT_SETS_CHANGED") then
    --    CleanUI_UpdateEquipmentSetItems();
    --end
end 

function CleanUI_GameMenuHookEvents()
    hooksecurefunc("GameMenuFrame_UpdateVisibleButtons", CleanUI_GameMenuFrame_UpdateVisibleButtons);

    hooksecurefunc(GameMenuFrame, "Show", CleanUI_GameMenuFrame_Show);
    hooksecurefunc(GameMenuFrame, "Hide", CleanUI_GameMenuFrame_Hide);
end

function CleanUI_GameMenuFrame_Show()
    UIFrameFadeOut(UIParent, 0.3, UIParent:GetAlpha(), 0.1);
    UIFrameFadeOut(ObjectiveTrackerFrame, 0.3, ObjectiveTrackerFrame:GetAlpha(), 0.1);
    UIFrameFadeOut(MinimapCluster, 0.3, MinimapCluster:GetAlpha(), 0.0);
    UIFrameFadeOut(Minimap, 0.3, Minimap:GetAlpha(), 0.1);
end

function CleanUI_GameMenuFrame_Hide()
    UIParent:SetAlpha(1);
    ObjectiveTrackerFrame:SetAlpha(1);
    MinimapCluster:SetAlpha(1);
    Minimap:SetAlpha(1);
end

function CleanUI_UpdateGameMenuBlackout()

    GameMenuFrame:SetParent(nil);
    GameMenuFrame:EnableMouse();
    
    if (not GameMenuFrame.blackout) then
        GameMenuFrame.blackout = CreateFrame("Frame", "GameMenuFrameBlackoutFrame", GameMenuFrame);
        GameMenuFrame.blackout:SetParent(GameMenuFrame);
        GameMenuFrame.blackout:SetFrameStrata("BACKGROUND");
        GameMenuFrame.blackout:EnableMouse();
        GameMenuFrame.blackout:SetPoint("TOPLEFT", UIParent, "TOPLEFT", 0, 0);
        GameMenuFrame.blackout:SetPoint("BOTTOMRIGHT", UIParent, "BOTTOMRIGHT", 0, 0);

        GameMenuFrame.blackout:SetScript("OnMouseUp", function()  
            HideUIPanel(GameMenuFrame);
        end);

        GameMenuFrame.blackout:SetScript("OnShow", function()  
            CleanUI_UpdateGameMenuInfo();
        end);

        GameMenuFrame.blackout.bgInfo = GameMenuFrame.blackout:CreateFontString("CUI_GameMenuInfo", "ARTWORK");
        GameMenuFrame.blackout.bgInfo:SetSize(10, 10);
        GameMenuFrame.blackout.bgInfo:SetFontObject("GameFontHighlightLeft");
        GameMenuFrame.blackout.bgInfo:SetJustifyH("CENTER");
        GameMenuFrame.blackout.bgInfo:SetPoint("TOPLEFT", GameMenuFrame.blackout, "BOTTOMLEFT", 10, 30);
        GameMenuFrame.blackout.bgInfo:SetPoint("BOTTOMRIGHT", GameMenuFrame.blackout, "BOTTOMRIGHT", -10, 0);
    end
end

function CleanUI_UpdateGameMenuInfo()
    local infotext = "";

    local version, build, date, tocversion = GetBuildInfo();
    infotext = infotext..string.format("Version: |c00FFFF66%s|r, Build: |c00FFFF66%s|r, Date: |c00FFFF66%s|r, TOC-Version: |c00FFFF66%s|r                  ", version, build, date, tocversion);

    local memoryInUse = gcinfo()
    infotext = infotext.."Memory in use: |c00FFFF66" .. string.format("%.2f", memoryInUse) .. " kb|r                  ";

    local framerate = GetFramerate();
    infotext = infotext.."Your current FPS is |c00FFFF66" .. floor(framerate) .. "fps|r                  ";

    local down, up, lagHome, lagWorld = GetNetStats();
    infotext = infotext.."Connection Lag: |c00FFFF66" .. lagHome .. " ms|r, World Lag: |c00FFFF66" .. lagWorld .. " ms|r"; 

    GameMenuFrame.blackout.bgInfo:SetText(infotext);
end

function CleanUI_UpdateGameMenu()
    CleanUI_RemoveTextures(GameMenuFrame);
    CleanUI_RemoveTextures(GameMenuFrame.Header);
    CleanUI_SetArtBackdrop(GameMenuFrame, 20);

    CleanUI_GameMenuFrame_UpdateVisibleButtons();

    GameMenuFrame.Border:Hide();
    GameMenuFrame:SetScale(0.75);

    -- Change button style

    CleanUI_ChangeGameMenuButton(GameMenuButtonHelp);
    CleanUI_ChangeGameMenuButton(GameMenuButtonStore);
    CleanUI_ChangeGameMenuButton(GameMenuButtonWhatsNew);

    CleanUI_ChangeGameMenuButton(GameMenuButtonOptions);
    CleanUI_ChangeGameMenuButton(GameMenuButtonUIOptions);
    CleanUI_ChangeGameMenuButton(GameMenuButtonKeybindings);
    CleanUI_ChangeGameMenuButton(GameMenuButtonMacros);
    CleanUI_ChangeGameMenuButton(GameMenuButtonAddons);

    CleanUI_ChangeGameMenuButton(CleanUIGameMenuButtonReloadUI);

    CleanUI_ChangeGameMenuButton(GameMenuButtonLogout, true);
    CleanUI_ChangeGameMenuButton(GameMenuButtonQuit, true);

    CleanUI_ChangeGameMenuButton(GameMenuButtonContinue);
end

function CleanUI_ChangeGameMenuButton(button, warn)
    CleanUI_RemoveUiButtonTexture(button);

    button:SetNormalTexture("interface\\Buttons\\UI-Listbox-Highlight2");
    button:GetNormalTexture():SetTexCoord(0.0, 1.0, 0.0, 1.0);
    button:GetNormalTexture():SetAlpha(0.2);

    button:SetHighlightTexture("interface\\Buttons\\UI-Listbox-Highlight");
    button:GetHighlightTexture():SetTexCoord(0.0, 1.0, 0.0, 1.0);

    if (warn) then
        button:GetHighlightTexture():SetDesaturated(1);
        button:GetHighlightTexture():SetVertexColor(0.6, 0.0, 0.0);
    else
        button:GetHighlightTexture():SetDesaturated(1);
        button:GetHighlightTexture():SetVertexColor(0.0, 0.6, 0.2);
    end
end

function CleanUI_GameMenuFrame_UpdateVisibleButtons()
    local height = GameMenuFrame:GetHeight() + 55;
    GameMenuFrame:SetHeight(height);
    
    GameMenuButtonLogout:ClearAllPoints();
    GameMenuButtonLogout:SetPoint("TOP", CleanUIGameMenuButtonReloadUI, "BOTTOM", 0, -10);
    
    if (not GameMenuFrame.info) then
        local title = GetAddOnMetadata("CleanUI", "Title");
        local version = GetAddOnMetadata("CleanUI", "Version");
    
        GameMenuFrame.info = GameMenuFrame:CreateFontString("GameMenuFrameInfo", "ARTWORK");
        GameMenuFrame.info:SetFontObject("CuiFont_ExtraSmall");
        GameMenuFrame.info:SetText("powered by |c333399ff"..title.."|r ("..version..")");
        GameMenuFrame.info:SetJustifyH("CENTER");
        GameMenuFrame.info:SetSize(180, 15);
        GameMenuFrame.info:ClearAllPoints();
        GameMenuFrame.info:SetPoint("BOTTOMLEFT", GameMenuFrame, "BOTTOMLEFT", 0, 4);
        GameMenuFrame.info:SetPoint("BOTTOMRIGHT", GameMenuFrame, "BOTTOMRIGHT", 0, 4);
    end
    
    if (true) then
        return;
    end
    
    -- rearange buttons
    
    GameMenuFrame:SetSize(370, 345);
    
    GameMenuButtonHelp:SetSize(145, 27);  
    GameMenuButtonHelp:ClearAllPoints();
    GameMenuButtonHelp:SetPoint("TOPLEFT", GameMenuFrame, "TOPLEFT", 20, -30); 
    
    GameMenuButtonStore:SetSize(145, 27); 
    GameMenuButtonWhatsNew:SetSize(145, 27); 
    
    if (not GameMenuFrame.settingsLabel) then
        GameMenuFrame.settingsLabel = GameMenuFrame:CreateFontString("GameMenuFrameInfo", "ARTWORK");
        GameMenuFrame.settingsLabel:SetFontObject("GameFontNormal");
        GameMenuFrame.settingsLabel:SetText(SETTINGS);
        GameMenuFrame.settingsLabel:SetJustifyH("LEFT");
        GameMenuFrame.settingsLabel:SetSize(145, 27);
        GameMenuFrame.settingsLabel:ClearAllPoints();
        GameMenuFrame.settingsLabel:SetPoint("TOP", GameMenuButtonWhatsNew, "BOTTOM", 0, -16);
    end
    
    GameMenuButtonOptions:SetSize(145, 27); 
    GameMenuButtonOptions:SetPoint("TOP", GameMenuFrame.settingsLabel, "BOTTOM", 0, -1);
    GameMenuButtonUIOptions:SetSize(145, 27); 
    
    GameMenuButtonKeybindings:ClearAllPoints();
    GameMenuButtonKeybindings:SetPoint("TOP", GameMenuButtonUIOptions, "BOTTOM", 0, -16);
    GameMenuButtonKeybindings:SetSize(145, 27); 
    GameMenuButtonMacros:SetSize(145, 27); 
    GameMenuButtonAddons:SetSize(145, 27); 
    
    CleanUIGameMenuButtonReloadUI:SetSize(145, 27);  
    CleanUIGameMenuButtonReloadUI:ClearAllPoints();
    CleanUIGameMenuButtonReloadUI:SetPoint("TOPRIGHT", GameMenuFrame, "TOPRIGHT", -20, -170); 
    
    GameMenuButtonLogout:SetSize(145, 27); 
    GameMenuButtonLogout:ClearAllPoints();
    GameMenuButtonLogout:SetPoint("TOP", CleanUIGameMenuButtonReloadUI, "BOTTOM", 0, -16);
    GameMenuButtonQuit:SetSize(145, 27); 
    GameMenuButtonContinue:SetSize(145, 27); 
end
