local MASQUE = LibStub("Masque", true);
local buttongroup = nil;

local BOTTOM_OFFSET = 350;

local RAID_FLARE_ICONS = {};
RAID_FLARE_ICONS[1] = "Interface\\AddOns\\CleanUI\\skins\\marker\\blue";
RAID_FLARE_ICONS[2] = "Interface\\AddOns\\CleanUI\\skins\\marker\\green";
RAID_FLARE_ICONS[3] = "Interface\\AddOns\\CleanUI\\skins\\marker\\purple";
RAID_FLARE_ICONS[4] = "Interface\\AddOns\\CleanUI\\skins\\marker\\red";
RAID_FLARE_ICONS[5] = "Interface\\AddOns\\CleanUI\\skins\\marker\\yellow";
RAID_FLARE_ICONS[6] = "Interface\\AddOns\\CleanUI\\skins\\marker\\orange";
RAID_FLARE_ICONS[7] = "Interface\\AddOns\\CleanUI\\skins\\marker\\silver";
RAID_FLARE_ICONS[8] = "Interface\\AddOns\\CleanUI\\skins\\marker\\white";

local microbutton_names = {
    "CharacterMicroButton",
    "SpellbookMicroButton",
    "TalentMicroButton",
    "AchievementMicroButton",
    "QuestLogMicroButton",

    "GuildMicroButton",
    "LFDMicroButton",
    "CollectionsMicroButton",
    "EJMicroButton",
    "StoreMicroButton",

    "MainMenuMicroButton",
    "HelpMicroButton",
}

local raidIconSnaped = false;
local raidFlareSnaped = false;

function CleanUI_InitMicroButtons()

    if (MASQUE) then
        buttongroup = MASQUE:Group("CleanUI", "Professions");
    end

    -- additional bars
    
    CleanUI_CreateRaidIconBar();
    CleanUI_CreateRaidFlareBar();
    CleanUI_CreateCraftBar();

    -- create micro button bar

    local buttonbar = CreateFrame("Frame", "CUI_MicroButtonFrame", UIParent, BackdropTemplateMixin and "BackdropTemplate");
    CleanUI_SetBackdrop(buttonbar);

    buttonbar:SetSize(300, 55);
    buttonbar:SetFrameStrata("BACKGROUND");
    
    buttonbar:SetPoint("RIGHT", UIParent, "BOTTOMLEFT", 5, BOTTOM_OFFSET);
    
    buttonbar:EnableMouse(true);
    buttonbar:HookScript("OnEnter", CleanUI_MicroButtonBar_OnEnter);
    buttonbar:HookScript("OnLeave", CleanUI_MicroButtonBar_OnLeave);

    -- create professions bar

    local professionsBar = CreateFrame("Frame", "CUI_MicroButtonFrameRaidBarProfessions", UIParent, BackdropTemplateMixin and "BackdropTemplate");
    CleanUI_SetBackdrop(professionsBar);

    professionsBar:SetSize(300, 40);
    professionsBar:SetFrameStrata("BACKGROUND");
    
    professionsBar:SetPoint("TOP", buttonbar, "BOTTOM", 0, 4);
    
    professionsBar:EnableMouse(true);
    professionsBar:HookScript("OnEnter", CleanUI_MicroButtonBar_OnEnter);
    professionsBar:HookScript("OnLeave", CleanUI_MicroButtonBar_OnLeave);
    
    CleanUI_CreateProfessionsButtons(professionsBar);
    
    -- create raid icon bar

    local raidbar = CreateFrame("Frame", "CUI_MicroButtonFrameRaidBar", UIParent, BackdropTemplateMixin and "BackdropTemplate");
    CleanUI_SetBackdrop(raidbar);
    
    raidbar:SetSize(169, 27);
    raidbar:SetFrameStrata("BACKGROUND");
    
    raidbar:ClearAllPoints();
    raidbar:SetPoint("BOTTOMLEFT", buttonbar, "TOPLEFT", 0, -4);
    
    raidbar:EnableMouse(true);
    raidbar:HookScript("OnEnter", CleanUI_MicroButtonBar_OnEnter);
    raidbar:HookScript("OnLeave", CleanUI_MicroButtonBar_OnLeave);
    
    -- create flare bar

    local flarebar = CreateFrame("Frame", "CUI_MicroButtonFrameFlareBar", UIParent, BackdropTemplateMixin and "BackdropTemplate");
    CleanUI_SetBackdrop(flarebar);
    
    flarebar:SetSize(135, 27);
    flarebar:SetFrameStrata("BACKGROUND");
    
    flarebar:ClearAllPoints();
    flarebar:SetPoint("BOTTOMRIGHT", buttonbar, "TOPRIGHT", 0, -4);
    
    flarebar:EnableMouse(true);
    flarebar:HookScript("OnEnter", CleanUI_MicroButtonBar_OnEnter);
    flarebar:HookScript("OnLeave", CleanUI_MicroButtonBar_OnLeave);
    
    -- relocate micro buttons

    hooksecurefunc("UpdateMicroButtons", CleanUI_RelocateMicroButtons);
    hooksecurefunc("MoveMicroButtons", CleanUI_RelocateMicroButtons);
    
    CleanUI_RelocateMicroButtons();
    
    SpellbookMicroButton:HookScript("OnEnter", CleanUI_MicroButton_OnEnter);
    hooksecurefunc("MicroButton_OnEnter", CleanUI_MicroButton_OnEnter);

    for ii=1, #(microbutton_names), 1 do
        local button = _G[microbutton_names[ii]];

        if (button) then
            button:HookScript("OnEnter", CleanUI_MicroButtonBar_OnEnter);
            button:HookScript("OnLeave", CleanUI_MicroButtonBar_OnLeave);

            button:SetFrameStrata("DIALOG");

        --SpellbookMicroButton.icon:SetTexCoord(0.05, 0.95, 0.05, 0.95);
        end
    end
    
    CleanUI_ResetRaidIconPosition()
    CleanUI_ResetRaidFlarePosition();
    CleanUI_ResetCraftBarPosition();
end

function CleanUI_MicroButton_OnEnter(self)
    GameTooltip:ClearAllPoints();
    GameTooltip:SetPoint("TOP", self, "BOTTOM", 0, 0);
end

function CleanUI_MicroButtonBar_OnEnter()
    if (InCombatLockdown()) then
        return;
    end

    if (raidIconSnaped or raidFlareSnaped) then
        CleanUI_MicroButtonBar_OnLeave();
        return;
    end

    CUI_MicroButtonFrame:SetPoint("RIGHT", UIParent, "BOTTOMLEFT", CUI_MicroButtonFrame:GetWidth() - 5, BOTTOM_OFFSET);
    
    CUI_MicroButtonFrameRaidBar:ClearAllPoints();
    CUI_MicroButtonFrameRaidBar:SetPoint("BOTTOMLEFT", CUI_MicroButtonFrame, "TOPLEFT", 0, -4);
    
    CUI_MicroButtonFrameFlareBar:ClearAllPoints();
    CUI_MicroButtonFrameFlareBar:SetPoint("BOTTOMRIGHT", CUI_MicroButtonFrame, "TOPRIGHT", 0, -4);
end

function CleanUI_MicroButtonBar_OnLeave()
    if (InCombatLockdown()) then
        return;
    end

    CUI_MicroButtonFrame:SetPoint("RIGHT", UIParent, "BOTTOMLEFT", 5, BOTTOM_OFFSET);
    
    CUI_MicroButtonFrameRaidBar:ClearAllPoints();
    CUI_MicroButtonFrameRaidBar:SetPoint("BOTTOMLEFT", CUI_MicroButtonFrame, "TOPLEFT", 0, -4);
    
    CUI_MicroButtonFrameFlareBar:ClearAllPoints();
    CUI_MicroButtonFrameFlareBar:SetPoint("BOTTOMRIGHT", CUI_MicroButtonFrame, "TOPRIGHT", 0, -4);
end

function CleanUI_RelocateMicroButtons()
    CharacterMicroButton:ClearAllPoints();
    CharacterMicroButton:SetPoint("LEFT", _G["CUI_MicroButtonFrame"], "LEFT", 7, 5);

    for ii=2, #(microbutton_names), 1 do
        local button = _G[microbutton_names[ii]];
        local buttonref = _G[microbutton_names[ii-1]];

        if (button and buttonref) then
            button:ClearAllPoints();
            button:SetPoint("LEFT", buttonref, "RIGHT", -2, 0);
        end
    end
end

function CleanUI_CreateProfessionsButtons(parent)
    local prof1, prof2, arch, fish, cook = GetProfessions();

    local buttons = {};
    local actions = {};

    CleanUI_AppendProfessionsButtons(prof1, buttons, actions);
    CleanUI_AppendProfessionsButtons(prof2, buttons, actions);
    CleanUI_AppendProfessionsButtons(arch, buttons, actions);
    CleanUI_AppendProfessionsButtons(fish, buttons, actions);
    CleanUI_AppendProfessionsButtons(cook, buttons, actions);

    -- professions

    SpellBookFrame.bookType = BOOKTYPE_PROFESSION;

    local parent = CUI_MicroButtonFrameRaidBarProfessions;
    local space = 4;

    CleanUI_CreateCookingFireButton(parent);

    for i, button in ipairs(buttons) do
        if (i == 1) then
            button:SetPoint("LEFT", parent, "LEFT", 8, 0);
        else
            button:SetPoint("LEFT", parent, "RIGHT", 4, 0);
        end

        parent = button;
    end

    -- actions
    
    local frame = _G["CUI_ChatCraftbar"];

    frame:SetSize(#actions * 30 + 10, 36);
    
    local parent = frame;

    for i, action in ipairs(actions) do
        action:SetParent(frame);
        action:SetSize(27, 27);

        if (i == 1) then
            action:SetPoint("LEFT", parent, "LEFT", 5, 0);
        else
            action:SetPoint("LEFT", parent, "RIGHT", 4, 0);
        end

        parent = action;
    end
end

function CleanUI_CreateCookingFireButton(parent)
    local id = 818; -- 818 -> cooking fire

    local button = CreateFrame("Button", nil, parent, "CUI_ProfessionActionButtonTemplate");
    button:RegisterForClicks("AnyUp");
    button.rangeTimer = -1;

    button:RegisterEvent("PLAYER_TARGET_CHANGED");
    button:RegisterEvent("ACTIONBAR_UPDATE_USABLE");
    button:RegisterEvent("ACTIONBAR_UPDATE_COOLDOWN");
    
    local spellname, rank, texture = GetSpellInfo(id);

    button:SetAttribute("type", "spell");
    button:SetAttribute("spell", spellname);
    button:SetAttribute("id", id);
    
    button:SetNormalTexture(nil);
    button:SetPushedTexture(nil);
    
    button:ClearAllPoints();
    button:SetPoint("RIGHT", parent, "RIGHT", -8, 0);

    button:SetSize(27, 27);

    button.icon:SetTexture(GetSpellTexture(spellname));
    button.icon:SetTexCoord(0.05, 0.95, 0.05, 0.95);

    if (MASQUE) then
        buttongroup:AddButton(button);
    end
end

function CleanUI_AppendProfessionsButtons(index, buttons, actions)
    if index then
		local name, texture, rank, maxRank, numSpells, spelloffset, skillLine, rankModifier, specializationIndex, specializationOffset, skillLineName = GetProfessionInfo(index);
    
        if numSpells <= 0 then
			--nothing to do
		elseif numSpells == 1 then
			CleanUI_AppendProfessionButton(index, buttons);
		else -- if numSpells >= 2 then
			CleanUI_AppendProfessionButton(index, buttons);
			CleanUI_AppendProfessionAction(index, actions);
		end
    
    end
end

function CleanUI_AppendProfessionButton(index, buttons)
    local name, texture, rank, maxRank, numSpells, spelloffset, skillLine, rankModifier, specializationIndex, specializationOffset, skillLineName = GetProfessionInfo(index);
    local spellIndex = spelloffset + 1;

	local texture = GetSpellBookItemTexture(spellIndex, BOOKTYPE_PROFESSION);
	local spellName, _, spellID = GetSpellBookItemName(spellIndex, BOOKTYPE_PROFESSION);
    
	if ( IsPassiveSpell(spellIndex, BOOKTYPE_PROFESSION) ) then
		return;
    end
    
    local containerFrame = CreateFrame("Frame", nil, CUI_MicroButtonFrameRaidBarProfessions);
    containerFrame:SetSize(27, 27);
    containerFrame.spellOffset = spelloffset;  

    containerFrame:EnableMouse(true);
    containerFrame:HookScript("OnEnter", CleanUI_MicroButtonBar_OnEnter);
    containerFrame:HookScript("OnLeave", CleanUI_MicroButtonBar_OnLeave);
    
    local button = CreateFrame("CheckButton", nil, containerFrame, "CUI_ProfessionButtonTemplate");
    button:SetID(1);

    button.skillName = name;
    button.spellOffset = spelloffset;
    button.skillLine = skillLine;
    button.specializationIndex = specializationIndex;
    button.specializationOffset = specializationOffset;

    button:SetPoint("TOPLEFT", containerFrame, "TOPLEFT", 0, 0);
    button:SetPoint("BOTTOMRIGHT", containerFrame, "BOTTOMRIGHT", 0, 0);

    button.iconTexture:SetTexture(texture);
    button.highlightTexture:SetTexture("Interface\\Buttons\\ButtonHilight-Square");
   
    local start, duration, enable = GetSpellCooldown(spellIndex, BOOKTYPE_PROFESSION);
	CooldownFrame_Set(button.cooldown, start, duration, enable);
    
    if ( enable == 1 ) then
		button.iconTexture:SetVertexColor(1.0, 1.0, 1.0);
	else
		button.iconTexture:SetVertexColor(0.4, 0.4, 0.4);
    end
    
    button.iconTexture:SetTexture(texture);
    button.iconTexture:SetTexCoord(0.05, 0.95, 0.05, 0.95);

    SpellButton_UpdateSelection(button);

    if (MASQUE) then
        buttongroup:AddButton(button);
    end
    
    tinsert(buttons, containerFrame);
end

function CleanUI_AppendProfessionAction(index, actions)
    local name, texture, rank, maxRank, numSpells, spelloffset, skillLine, rankModifier, specializationIndex, specializationOffset, skillLineName = GetProfessionInfo(index);
    local spellIndex = spelloffset + 2;

	local texture = GetSpellBookItemTexture(spellIndex, BOOKTYPE_PROFESSION);
	local spellName, _, spellID = GetSpellBookItemName(spellIndex, BOOKTYPE_PROFESSION);
    
	if ( IsPassiveSpell(spellIndex, BOOKTYPE_PROFESSION) ) then
		return;
    end
    
    local containerFrame = CreateFrame("Frame", nil, CUI_MicroButtonFrameRaidBarProfessions);

    local button = CreateFrame("Button", nil, containerFrame, "CUI_ProfessionActionTemplate");

    button:RegisterForClicks("AnyUp");
    button.rangeTimer = -1;

    button:RegisterEvent("PLAYER_TARGET_CHANGED");
    button:RegisterEvent("ACTIONBAR_UPDATE_USABLE");
    button:RegisterEvent("ACTIONBAR_UPDATE_COOLDOWN");
    
    local spellname, rank, texture = GetSpellInfo(spellID);

    button:SetAttribute("type", "spell");
    button:SetAttribute("spell", spellname);
    button:SetAttribute("id", spellID);
    
    button.icon:SetTexture(texture);
    button.icon:SetTexCoord(0.1, 0.9, 0.1, 0.9);
    
    button:SetNormalTexture(nil);
    button:SetPushedTexture(nil);
   
    local start, duration, enable = GetSpellCooldown(spellIndex, BOOKTYPE_PROFESSION);
	CooldownFrame_Set(button.cooldown, start, duration, enable);

    button:SetPoint("TOPLEFT", containerFrame, "TOPLEFT", 0, 0);
    button:SetPoint("BOTTOMRIGHT", containerFrame, "BOTTOMRIGHT", 0, 0);
    
    tinsert(actions, containerFrame);
end

function CleanUI_MicroButtons_Professions_OnClick(self)
    local slot = self:GetID() + self:GetParent().spellOffset;

    if (self:GetID() == 1) then
        -- castable -> open frame
        CastSpell(slot, BOOKTYPE_PROFESSION);
    end

    SpellButton_UpdateSelection(self);
end

function CleanUI_CreateRaidIconBar()
    local raidicons = CreateFrame("Frame", "CUI_ChatRaidIcons", UIParent);
    raidicons:SetSize(160, 23);
    raidicons:SetScale(1);
    raidicons:SetFrameStrata("DIALOG");

    for i=1, 8, 1 do

        local posx = -6-(i-1) * 16;

        local btn = CreateFrame("Button", "CUI_CHAT_RAIDICON_"..i, raidicons, "UIPanelButtonTemplate");

        btn:SetNormalTexture(CUI_RAIDICON_FILE);
        btn:SetPushedTexture(CUI_RAIDICON_FILE);
        btn:GetNormalTexture():SetTexCoord(CUI_RAIDICONS[i].left, CUI_RAIDICONS[i].right, CUI_RAIDICONS[i].top, CUI_RAIDICONS[i].bottom);
        btn:GetPushedTexture():SetTexCoord(CUI_RAIDICONS[i].left, CUI_RAIDICONS[i].right, CUI_RAIDICONS[i].top, CUI_RAIDICONS[i].bottom);
        CleanUI_RemoveUiButtonTexture(btn);

        btn:RegisterForClicks("AnyUp");

        btn:SetWidth(13);
        btn:SetHeight(13);
        btn:ClearAllPoints();
        btn:SetPoint("RIGHT", raidicons, "RIGHT", posx, 0);

        btn:SetAttribute("index", i);

        btn:HookScript("OnClick", CleanUI_RaidIconClicked);
        btn:HookScript("OnEnter", CleanUI_MicroButtonBar_OnEnter);
        btn:HookScript("OnLeave", CleanUI_MicroButtonBar_OnLeave);
    end
    
    HelpOpenWebTicketButton:HookScript("OnEnter", CleanUI_MicroButtonBar_OnEnter);
    HelpOpenWebTicketButton:HookScript("OnLeave", CleanUI_MicroButtonBar_OnLeave);

    -- rc button
    local rcbutton = CreateFrame("Button", "CUI_CHAT_RCBUTTON", raidicons, "UIPanelButtonTemplate");

    rcbutton:SetNormalTexture("interface\\RaidFrame\\ReadyCheck-Ready");
    rcbutton:SetPushedTexture("interface\\RaidFrame\\ReadyCheck-Ready");
    rcbutton:SetHighlightTexture("interface\\RaidFrame\\ReadyCheck-Ready");
    CleanUI_RemoveUiButtonTexture(rcbutton);

    rcbutton:GetNormalTexture():SetTexCoord(0, 1, 0, 1);
    rcbutton:GetPushedTexture():SetTexCoord(0, 1, 0, 1);
    rcbutton:GetHighlightTexture():SetTexCoord(0, 1, 0, 1);

    rcbutton:SetWidth(18);
    rcbutton:SetHeight(18);
    rcbutton:ClearAllPoints();
    rcbutton:SetPoint("LEFT", raidicons, "LEFT", 3, 0);
    rcbutton:HookScript("OnClick", function() DoReadyCheck(); end);
    
    rcbutton:HookScript("OnEnter", CleanUI_MicroButtonBar_OnEnter);
    rcbutton:HookScript("OnLeave", CleanUI_MicroButtonBar_OnLeave);
end

function CleanUI_CreateRaidFlareBar()
    local raidflares = CreateFrame("Frame", "CUI_ChatRaidFlares", UIParent);
    raidflares:SetSize(130, 23);
    raidflares:SetScale(1);
    raidflares:SetFrameStrata("DIALOG");

    for i=1, 8, 1 do
        local posx = -6-(i-1) * 15;

        local btn = CreateFrame("Button", "CUI_CHAT_RAIDFLARE_"..i, raidflares, "SecureActionButtonTemplate");

        btn:SetNormalTexture(RAID_FLARE_ICONS[i]);
        btn:SetPushedTexture(RAID_FLARE_ICONS[i]);

        btn:RegisterForClicks("AnyUp");
        btn:SetAttribute("type1", "macro");
        btn:SetAttribute("macrotext1", "/worldmarker "..i);
        btn:SetAttribute("type2", "macro");
        btn:SetAttribute("macrotext2", "/clearworldmarker "..i);

        btn:SetWidth(13);
        btn:SetHeight(13);
        btn:ClearAllPoints();
        btn:SetPoint("RIGHT", raidflares, "RIGHT", posx, 0);

        btn:SetAttribute("index", i);
        
        btn:HookScript("OnEnter", CleanUI_MicroButtonBar_OnEnter);
        btn:HookScript("OnLeave", CleanUI_MicroButtonBar_OnLeave);
    end
end

function CleanUI_CreateCraftBar()
    local frame = CreateFrame("Frame", "CUI_ChatCraftbar", UIParent, BackdropTemplateMixin and "BackdropTemplate");
    frame:SetSize(130, 23);
    frame:SetScale(1);
    frame:SetFrameStrata("DIALOG");

    CleanUI_SetBackdrop(frame);
end

function CleanUI_RaidIconSnap()
    local frame = _G["CUI_ChatRaidIcons"];

    icons_posx = frame:GetLeft();
    icons_posy = frame:GetBottom();

    local x, y = GetCursorPosition();
    local scale = UIParent:GetEffectiveScale();

    x = x / scale;
    y = y / scale;

    frame:ClearAllPoints();
    frame:SetPoint("CENTER", UIParent, "BOTTOMLEFT", x, y);
    
    raidIconSnaped = true;

    CleanUI_SetBackdrop(frame);
end

function CleanUI_ResetRaidIconPosition()
    local frame = _G["CUI_ChatRaidIcons"];
    local borderframe = _G["CUI_MicroButtonFrameRaidBar"];
    
    if (borderframe) then
        frame:ClearAllPoints();
        frame:SetPoint("CENTER", borderframe, "CENTER", 0, 0);   
    
        --frame:SetBackdrop(nil);  
    else
        frame:ClearAllPoints();
        frame:SetPoint("CENTER", UIParent, "BOTTOMLEFT", 100000, 0);
    end
    
    raidIconSnaped = false;
end

function CleanUI_RaidIconClicked(self, button, down)

    local index = self:GetAttribute("index");

    if (not UnitExists("target") or IsAltKeyDown()) then
        CleanUI_ChatInsertText("{rt"..index.."}");
    elseif (button == "LeftButton") then
        SetRaidTarget("target", index);
    elseif (button == "RightButton") then
        SetRaidTarget("target", 0);
    end
end

function CleanUI_ChatInsertText(text)
    ChatMenu:Hide();

    if (not DEFAULT_CHAT_FRAME.editBox:IsShown()) then
        DEFAULT_CHAT_FRAME.editBox:Show();
    end

    DEFAULT_CHAT_FRAME.editBox:SetFocus()
    DEFAULT_CHAT_FRAME.editBox:Insert(text);
end

function CleanUI_RaidFlareSnap()
    if (InCombatLockdown()) then
        return;
    end

    local doSnap = true;

    if (IsInRaid()) then
        if ( UnitIsGroupLeader("player") or UnitIsGroupAssistant("player") ) then
            doSnap = true;
        else
            doSnap = false;
        end
    end

    if (not doSnap) then
        return;
    end

    local frame = _G["CUI_ChatRaidFlares"];

    icons_posx = frame:GetLeft();
    icons_posy = frame:GetBottom();

    local x, y = GetCursorPosition();
    local scale = UIParent:GetEffectiveScale();

    x = x / scale;
    y = y / scale;

    frame:ClearAllPoints();
    frame:SetPoint("CENTER", UIParent, "BOTTOMLEFT", x, y);
    
    raidFlareSnaped = true;

    CleanUI_SetBackdrop(frame);
end

function CleanUI_ResetRaidFlarePosition()
    if (InCombatLockdown()) then
        return;
    end
    
    local frame = _G["CUI_ChatRaidFlares"];
    local borderframe = _G["CUI_MicroButtonFrameFlareBar"];
    
    if (borderframe) then
        frame:ClearAllPoints();
        frame:SetPoint("CENTER", borderframe, "CENTER", 0, 0);   
    
        frame.backdropInfo = nil; 
    else
        frame:ClearAllPoints();
        frame:SetPoint("CENTER", UIParent, "BOTTOMLEFT", 100000, 0);
    end
    
    raidFlareSnaped = false;
end

function CleanUI_CraftBarSnap()
    if (InCombatLockdown()) then
        return;
    end

    local doSnap = true;

    if (IsInRaid()) then
        if ( UnitIsGroupLeader("player") or UnitIsGroupAssistant("player") ) then
            doSnap = true;
        else
            doSnap = false;
        end
    end

    if (not doSnap) then
        return;
    end

    local frame = _G["CUI_ChatCraftbar"];

    icons_posx = frame:GetLeft();
    icons_posy = frame:GetBottom();

    local x, y = GetCursorPosition();
    local scale = UIParent:GetEffectiveScale();

    x = x / scale;
    y = y / scale;

    frame:ClearAllPoints();
    frame:SetPoint("CENTER", UIParent, "BOTTOMLEFT", x, y);
end

function CleanUI_ResetCraftBarPosition()
    if (InCombatLockdown()) then
        return;
    end
    
    local frame = _G["CUI_ChatCraftbar"];
    frame:ClearAllPoints();
    frame:SetPoint("CENTER", UIParent, "BOTTOMLEFT", -100000, 0);
end
