petTypeFilterButtons = {};

-- saved data
CleanUIPetData = CleanUIPetData or {};
CleanUIBattlePetGroups = CleanUIBattlePetGroups or {};
local petStats = {};

-- move groups
local function VERSION_UPDATE()
    if (CleanUIBattlePetGroups.groups) then
        return;
    end
    
    local groups = {}
    
    for i, group in ipairs(CleanUIBattlePetGroups) do
        tinsert(groups, group);
    end
    
    CleanUIBattlePetGroups = {};
    CleanUIBattlePetGroups.groups = groups;
end

StaticPopupDialogs["CUI_BATTLEPET_GROUPS_ADD_TITLE_POPUP"] = {
    text = NAME,
    button1 = SAVE,
    button2 = CANCEL,
    OnShow = function (self, data)
        self.editBox:SetText(data or "nil")
    end,
    OnAccept = function (self, data, data2)
        CleanUI_AddPetBattleGroupWithName(self.editBox:GetText())
    end,
    hasEditBox = true,
    timeout = 0,
    whileDead = true,
    hideOnEscape = true,
}

StaticPopupDialogs["CUI_BATTLEPET_GROUPS_RENAME_TITLE_POPUP"] = {
    text = BATTLE_PET_RENAME,
    button1 = SAVE,
    button2 = CANCEL,
    OnShow = function (self, data)
        self.editBox:SetText(data or "nil")
    end,
    OnAccept = function (self, data, data2)
        CleanUI_RenamePetBattleGroupWithName(self.editBox:GetText())
    end,
    hasEditBox = true,
    timeout = 0,
    whileDead = true,
    hideOnEscape = true,
}

function CleanUI_InitPets()
    VERSION_UPDATE();

    CleanUI_RegisterPetsEvents();

    if (C_PetBattles.IsInBattle()) then
        CleanUI_Pets_UpdatePetBattleInfo();
    end
end

function CleanUI_RegisterPetsEvents()
    local frame = CreateFrame("Frame");

    frame:RegisterEvent("ADDON_LOADED");
    frame:RegisterEvent("PET_JOURNAL_PETS_HEALED");
    frame:RegisterEvent("PET_JOURNAL_PET_RESTORED");
    frame:RegisterEvent("PET_JOURNAL_PET_REVOKED");


	frame:RegisterEvent("PET_BATTLE_OPENING_START");
	frame:RegisterEvent("PET_BATTLE_CLOSE");

	frame:RegisterEvent("PET_BATTLE_PET_ROUND_PLAYBACK_COMPLETE");
	frame:RegisterEvent("PET_BATTLE_PET_CHANGED");
	frame:RegisterEvent("PET_BATTLE_XP_CHANGED");
	frame:RegisterEvent("PET_BATTLE_ACTION_SELECTED");
	frame:RegisterEvent("PET_BATTLE_PET_TYPE_CHANGED");
	frame:RegisterEvent("PET_BATTLE_OVERRIDE_ABILITY");

    frame:HookScript("OnEvent", CleanUI_PetsOnEvent);
end

function CleanUI_PetsOnEvent(self, event, ...)
    if (event == "ADDON_LOADED") then
        local addonName = ...;
        if ( addonName == "Blizzard_Collections") then
            CleanUI_Pets_ChangeUI();
            CleanUI_Pets_UpdatePetData();
            CleanUI_PetsHookEvents();
            CleanUI_InitPetBattleGroups();
        end
    end

    if (event == "PET_JOURNAL_PETS_HEALED" or event == "PET_JOURNAL_PET_RESTORED" or event == "PET_JOURNAL_PET_REVOKED") then
        CleanUI_UpdatePetBattleGroups();
    end

    if (event == "PET_BATTLE_PET_ROUND_PLAYBACK_COMPLETE" or 
        event == "PET_BATTLE_PET_CHANGED" or 
        event == "PET_BATTLE_XP_CHANGED" or 
        event == "PET_BATTLE_ACTION_SELECTED" or 
        event == "PET_BATTLE_PET_TYPE_CHANGED" or 
        event == "PET_BATTLE_OVERRIDE_ABILITY") then
        CleanUI_Pets_UpdatePetBattleInfo();
    end

    if (event == "PET_BATTLE_CLOSE") then
        CleanUI_Pets_HideAbilityTooltip();
    end
end

function CleanUI_Pets_UpdatePetData()  
    local numPets, numOwned = C_PetJournal.GetNumPets()

    for index = 1, numPets do
        local petID, speciesID, owned, customName, level, favorite, isRevoked, speciesName, icon, petType, companionID, tooltip, description, isWild, canBattle, isTradeable, isUnique, obtainable = C_PetJournal.GetPetInfoByIndex(index);

        if (speciesName) then
            CleanUIPetData[speciesName] = {
                petID=petID,
                speciesID=speciesID,
                owned=owned, 
                customName=customName, 
                level=level, 
                favorite=favorite, 
                isRevoked=isRevoked, 
                speciesName=speciesName, 
                icon=icon, 
                petType=petType, 
                companionID=companionID, 
                tooltip=tooltip, 
                description=description, 
                isWild=isWild, 
                canBattle=canBattle, 
                isTradeable=isTradeable, 
                isUnique=isUnique, 
                obtainable=obtainable
            };
        end
    end

end

function CleanUI_PetsHookEvents()   
    hooksecurefunc(C_PetJournal, "SetPetTypeFilter", CleanUI_Pets_UpdateFilterButtons);
    hooksecurefunc(C_PetJournal, "SetAllPetTypesChecked", CleanUI_Pets_UpdateFilterButtons);

    hooksecurefunc("PetJournal_UpdatePetList", CleanUI_PetJournal_UpdatePetList);
    hooksecurefunc("PetJournal_UpdatePetLoadOut", CleanUI_PetJournal_UpdatePetLoadOut);

    PetJournalListScrollFrame:HookScript("OnVerticalScroll", CleanUI_PetJournal_UpdatePetList)
    PetJournalListScrollFrame:HookScript("OnMouseWheel", CleanUI_PetJournal_UpdatePetList)
end

------------------------------------------------------------------------------------------------
-- FILTER BUTTONS
------------------------------------------------------------------------------------------------

local function createFilterButton(id, type)
    local filterButton = CreateFrame("CheckButton", nil, PetJournal.LeftInset, "SecureFrameTemplate");
    filterButton:SetSize(22, 22);
    filterButton:SetNormalTexture("Interface\\ICONS\\Icon_PetFamily_"..PET_TYPE_SUFFIX[id]);
    filterButton:GetNormalTexture():SetTexCoord(0.08, 0.92, 0.08, 0.92);
    filterButton:SetPoint("BOTTOMLEFT", PetJournal.listScroll, "TOPLEFT", 25 * (id - 1) + 3, 4);
    filterButton:SetAlpha(1);
    filterButton:SetID(id);

    filterButton:HookScript("OnEnter", CleanUI_Pets_OnButtonEnter);
    filterButton:HookScript("OnLeave", CleanUI_Pets_OnButtonLeave);

    filterButton:SetScript("OnClick", CleanUI_Pets_FilterClicked);

    return filterButton;
end

local function petBattlesCheatSheet(id)
    if (id == 1) then return 2, 5, 8, 4; end
    if (id == 2) then return 6, 3, 4, 1; end
    if (id == 3) then return 9, 8, 2, 6; end
    if (id == 4) then return 1, 2, 9, 5; end
    if (id == 5) then return 4, 7, 1, 8; end
    if (id == 6) then return 3, 9, 10, 2; end
    if (id == 7) then return 10, 10, 5, 9; end
    if (id == 8) then return 5, 1, 3, 10; end
    if (id == 9) then return 7, 4, 6, 3; end
    if (id == 10) then return 8, 6, 7, 7; end

    return 1, 1, 1, 1;
end

function CleanUI_Pets_OnButtonEnter(self)
    GameTooltip_SetDefaultAnchor(GameTooltip, self);
    GameTooltip:SetOwner(self, "ANCHOR_RIGHT");

    GameTooltip:SetText(_G["BATTLE_PET_NAME_"..self:GetID()], 1.0, 1.0, 1.0);

    local bonusStrike, bonusDefense, weakStrike, weakDefense = petBattlesCheatSheet(self:GetID());
    
    GameTooltip:AddLine(" ");
    GameTooltip:AddLine("Use Against", 0.0, 1.0, 0.0);
    GameTooltip:AddDoubleLine(_G["BATTLE_PET_NAME_"..bonusStrike], "Bonus Strike (+50%)", 1.0, 1.0, 1.0, 0.0, 1.0, 0.0);
	GameTooltip:AddTexture("Interface\\ICONS\\Icon_PetFamily_"..PET_TYPE_SUFFIX[bonusStrike]);
    GameTooltip:AddDoubleLine(_G["BATTLE_PET_NAME_"..bonusDefense], "Bonus Defense (+33%)", 1.0, 1.0, 1.0, 0.0, 1.0, 0.0);
    GameTooltip:AddTexture("Interface\\ICONS\\Icon_PetFamily_"..PET_TYPE_SUFFIX[bonusDefense]);
    
    GameTooltip:AddLine(" ");
    GameTooltip:AddLine("Don't Use Against", 1.0, 0.0, 0.0);
    GameTooltip:AddDoubleLine(_G["BATTLE_PET_NAME_"..weakStrike], "Weak Strike (-33%)", 1.0, 1.0, 1.0, 1.0, 0.0, 0.0);
	GameTooltip:AddTexture("Interface\\ICONS\\Icon_PetFamily_"..PET_TYPE_SUFFIX[weakStrike]);
    GameTooltip:AddDoubleLine(_G["BATTLE_PET_NAME_"..weakDefense], "Weak Defense (-50%)", 1.0, 1.0, 1.0, 1.0, 0.0, 0.0);
	GameTooltip:AddTexture("Interface\\ICONS\\Icon_PetFamily_"..PET_TYPE_SUFFIX[weakDefense]);

    GameTooltip:Show();
end

function CleanUI_Pets_OnButtonLeave(self)
    GameTooltip:FadeOut();
end

function CleanUI_Pets_ChangeUI()  
    if (PetJournal.cui_done) then
        return;
    end

    PetJournal.listScroll:SetPoint("TOPLEFT", PetJournal.LeftInset, "TOPLEFT", 3, -63);

    local button;
    for index = 1, 10 do
        button = createFilterButton(index);
        tinsert(petTypeFilterButtons, button);
    end

    PetJournal.cui_done = true;
    
    CleanUI_Pets_UpdateFilterButtons();
end

local function updateFilterButtonAlpha(button)
    local alpha = 1.0;
    if (not C_PetJournal.IsPetTypeChecked(button:GetID())) then
        alpha = 0.3;
    end

    button:SetAlpha(alpha);
end

function CleanUI_Pets_UpdateFilterButtons()
    if (not PetJournal.cui_done) then
        return;
    end

    for i, button in ipairs(petTypeFilterButtons) do
        updateFilterButtonAlpha(button);
    end
end

function CleanUI_Pets_FilterClicked(self)
    local checked = C_PetJournal.IsPetTypeChecked(self:GetID());
    C_PetJournal.SetPetTypeFilter(self:GetID(), not checked);

    if (IsAltKeyDown()) then
        for index = 1, 10 do
            C_PetJournal.SetPetTypeFilter(index, not checked);
        end
    end
end

function CleanUI_PetJournal_UpdatePetList()
    CleanUI_PetJournal_UpdatePetCount();
    CleanUI_PetJournal_UpdateCompanionListButtons();

    local scrollFrame = PetJournal.listScroll;
    local offset = HybridScrollFrame_GetOffset(scrollFrame);
    local petButtons = scrollFrame.buttons;
    local pet, index;

    local numPets, numOwned = C_PetJournal.GetNumPets();

    for i = 1,#petButtons do
        pet = petButtons[i];
        pet.name:SetWidth(125);

        if (not pet.cuihealth) then
            pet.cuihealth = CreateFrame("Frame", pet:GetName().."CUI_HEATH", pet, "CleanUIPetInfoHealthTemplate");
            pet.cuihealth:ClearAllPoints();
            pet.cuihealth:SetPoint("TOPRIGHT", pet, "TOPRIGHT", -10, -5);
            pet.cuihealth:SetScale(0.75);
        end

        if (not pet.sourceText) then
            pet.sourceText = pet:CreateFontString(name, "OVERLAY");
            pet.sourceText:SetFontObject("GameFontNormalTiny");
            pet.sourceText:SetJustifyH("RIGHT");
            pet.sourceText:ClearAllPoints();
            pet.sourceText:SetPoint("BOTTOMRiGHT", pet, "BOTTOMRiGHT", -10, 5);
        end

        index = offset + i;
        if index <= numPets then
            local petID, speciesID, isOwned, customName, level, favorite, isRevoked, name, icon, petType, creatureID, sourceText, description, isWildPet, canBattle = C_PetJournal.GetPetInfoByIndex(index);
            
            if isOwned then
                local health, maxHealth, attack, speed, rarity = C_PetJournal.GetPetStats(petID);
                local speciesID, customName, level, xp, maxXp, displayID, isFavorite, name, icon, petType, creatureID = C_PetJournal.GetPetInfoByPetID(petID);

                if (health < maxHealth) then
                    pet.cuihealth:Show();
                    pet.cuihealth.healthValue:SetFormattedText(PET_BATTLE_CURRENT_HEALTH_FORMAT, health, maxHealth);
                    pet.cuihealth.healthBar:SetMinMaxValues(0, maxHealth);
                    pet.cuihealth.healthBar:SetValue(health);
                else
                    pet.cuihealth:Hide();
                end

                pet.sourceText:SetText("");
            else
                pet.cuihealth:Hide();

                pet.sourceText:SetText(sourceText);
            end
        end
    end
end

function CleanUI_PetJournal_UpdatePetCount()
    local pets = {};

    petStats = {};

    local numUnique = 0;

    local numPets, numOwned = C_PetJournal.GetNumPets();

    local petID, speciesID, isOwned, customName, level, favorite, isRevoked, name, icon, petType, creatureID, sourceText, description, isWildPet, canBattle;
    local health, maxHealth, attack, speed, rarity;
    for i = 1, numPets do
        petID, speciesID, isOwned, customName, level, favorite, isRevoked, name, icon, petType, creatureID, sourceText, description, isWildPet, canBattle = C_PetJournal.GetPetInfoByIndex(i);
        if (isOwned) then
            health, maxHealth, attack, speed, rarity = C_PetJournal.GetPetStats(petID);

            petStats[rarity] = (petStats[rarity] or 0) + 1;

            if (speciesID and not pets[speciesID]) then
                numUnique = numUnique + 1;
                pets[speciesID] = speciesID;
            end
        else
            petStats[0] = (petStats[0] or 0) + 1;
        end
    end

    PetJournal.PetCount.Count:SetText(numOwned.." ("..numUnique..")");
    PetJournal.PetCount:SetSize(145, 20);
end

function CleanUI_PetJournal_UpdateCompanionListButtons()
    local scrollFrame = PetJournal.listScroll;
    local offset = HybridScrollFrame_GetOffset(scrollFrame);
    local petButtons = scrollFrame.buttons;
    local pet, index;

    local numPets, numOwned = C_PetJournal.GetNumPets();

    local summonedPetID = C_PetJournal.GetSummonedPetGUID();

    local petID, speciesID, isOwned, customName, level, favorite, isRevoked, name, icon, petType, creatureID, sourceText, description, isWildPet, canBattle;
    local health, maxHealth, attack, speed, rarity;
    local border;
    for i = 1,#petButtons do
        pet = petButtons[i];
        
        index = offset + i;
        if index <= numPets then
            petID, speciesID, isOwned, customName, level, favorite, isRevoked, name, icon, petType, creatureID, sourceText, description, isWildPet, canBattle = C_PetJournal.GetPetInfoByIndex(index);

            if (isOwned) then
                health, maxHealth, attack, speed, rarity = C_PetJournal.GetPetStats(petID);
                pet.name:SetTextColor(ITEM_QUALITY_COLORS[rarity-1].r, ITEM_QUALITY_COLORS[rarity-1].g, ITEM_QUALITY_COLORS[rarity-1].b);
            else
                pet.name:SetTextColor(0.9, 0.9, 0.0);
            end
        end
    end
end

------------------------------------------------------------------------------------------------
-- PET BATTLE GROUPS
------------------------------------------------------------------------------------------------

function CleanUI_InitPetBattleGroups()
    local button = CreateFrame("Button", "CUIPetBattleGroupsButton", PetJournal, "UIPanelButtonTemplate");
    button:SetWidth(100);
    button:SetHeight(22);
    button:SetPoint("TOPRIGHT", PetJournal, "TOPRIGHT", -6, -230);
    button:SetText(GROUPS);
    button:SetScript("OnClick", CleanUI_TogglePetBattleGroups);
    button:SetFrameLevel(button:GetFrameLevel() + 1);

    local groupsFrame = CreateFrame("Frame", "CleanUIBattlePetGroupsFrame", PetJournal, "CleanUIBattlePetGroupsFrameTemplate");
    
    CleanUI_SetArtBackdrop(groupsFrame, 0, 0, 0, 5);

    groupsFrame:SetSize(265, 605);
    groupsFrame:ClearAllPoints();
    groupsFrame:SetPoint("TOPLEFT", PetJournal, "TOPRIGHT", 4, 0);

    -- create gui
    HybridScrollFrame_OnLoad(CleanUIBattlePetGroupsFrame.scrollFrame);
    CleanUIBattlePetGroupsFrame.scrollFrame.update = CleanUI_BattlePetGroups_OnUpdate;
    HybridScrollFrame_CreateButtons(CleanUIBattlePetGroupsFrame.scrollFrame, "CleanUIBattlePetGroupsEntryTemplate");
end

function CleanUI_TogglePetBattleGroups()
    if (CleanUIBattlePetGroupsFrame:IsVisible()) then
        CleanUIBattlePetGroupsFrame:Hide();
    else
        CleanUIBattlePetGroupsFrame:Show();
        CleanUI_BattlePetGroups_OnUpdate();
    end
end

function CleanUI_UpdatePetBattleGroups()
    if (CleanUIBattlePetGroupsFrame:IsVisible()) then
        CleanUI_BattlePetGroups_OnUpdate();
    end
end

function CleanUI_BattlePetGroups_OnUpdate()
    if (not CleanUIBattlePetGroupsFrame) then
        return;
    end

    --update ui stats
    local groupsActive = 1;
    for p=1, 3 do
        _, _, _, _, locked = C_PetJournal.GetPetLoadOutInfo(p);
        if (locked) then
            groupsActive = nil;
        end
    end

    if (groupsActive) then
        CUIPetBattleGroupsButton:Enable();
    else
        CUIPetBattleGroupsButton:Disable();
    end

    -- update group frame
    CleanUIBattlePetGroupsFrame.scrollFrame.highlightFrame:Hide();

    local buttons = CleanUIBattlePetGroupsFrame.scrollFrame.buttons;

    local numEntries = #CleanUIBattlePetGroups.groups;
    local numButtons = #buttons;

    local scrollOffset = HybridScrollFrame_GetOffset(CleanUIBattlePetGroupsFrame.scrollFrame);
    local buttonHeight = buttons[1]:GetHeight();
    local displayedHeight = 0;

    local entry;
    local data;
    local editBox;

    for i=1, numButtons do
        local entryIndex = i + scrollOffset;
        entry = buttons[i];

        if ( entryIndex <= numEntries ) then
            data = CleanUIBattlePetGroups.groups[entryIndex];
            entry.data = data;

            entry:Show();
            entry.index = entryIndex;

            -- button data
            entry:SetText(data.name);

            if (UnitExists("target") and data.name == UnitName("target")) then
                entry:SetText(ORANGE_FONT_COLOR_CODE.."** "..data.name.." **"..FONT_COLOR_CODE_CLOSE);
            end

            -- info text
            editBox = _G[entry:GetName().."EditBox"];
            editBoxNoInfo = _G[entry:GetName().."EditBoxNoInfo"];
            if (data.infoText and data.infoText ~= "") then
                editBox:SetText(data.infoText);
                editBoxNoInfo:Hide();
            else
                editBox:SetText("");
                editBoxNoInfo:Show();
            end

            editBoxNoInfo:SetAlpha(0.5);

			editBox:SetCursorPosition(0);

            -- pet data
            local id, health, maxHealth, power, speed, rarity;
            local speciesID, customName, level, xp, maxXp, displayID, isFavorite, name, icon, petType, creatureID, sourceText, description, isWild, canBattle, tradable, unique, obtainable;
            local pet;
            for p=1, 3 do
                id = data.pet[p].petGUID;
                
                if (id) then
                    health, maxHealth, power, speed, rarity = C_PetJournal.GetPetStats(id);

                    speciesID, customName, level, xp, maxXp, displayID, isFavorite, name, icon, petType, creatureID, sourceText, description, isWild, canBattle, tradable, unique, obtainable = C_PetJournal.GetPetInfoByPetID(id);
    
                    pet = _G[entry:GetName().."Pet"..p];
    
                    rarity = rarity or 1;
                
                    pet.PetInfo.icon:SetTexture(icon);
                    pet.PetInfo.qualityBorder:SetDesaturated(nil);
                    pet.PetInfo.qualityBorder:SetVertexColor(ITEM_QUALITY_COLORS[rarity-1].r, ITEM_QUALITY_COLORS[rarity-1].g, ITEM_QUALITY_COLORS[rarity-1].b);

                    if (level < 25) then
                        pet.PetInfo.level:SetText(level);
                        pet.PetInfo.level:Show();
                        pet.PetInfo.levelBG:Show();
                    else
                        pet.PetInfo.level:Hide();
                        pet.PetInfo.levelBG:Hide();
                    end

                    if (isFavorite) then
                        pet.PetInfo.favorite:Show();
                    else
                        pet.PetInfo.favorite:Hide();
                    end
    
                    pet.PetInfo.typeIcon:SetTexture(GetPetTypeTexture(petType));
                    pet.PetInfo.isDead:SetShown((health or 0) <= 0);
    
                    -- tooltip info
                    pet.speciesName = name;
                    pet.sourceText = sourceText;
                    pet.tradable = tradable;
                    pet.unique = unique;
    
                    if ( description and description ~= "" ) then
                        pet.description = format([["%s"]], description);
                    else
                        pet.description = nil;
                    end
                end
            end

            -- this isn't a header, hide the header textures
            entry:SetNormalTexture("");
            entry:SetHighlightTexture("");

            if (entryIndex == selectedIndex) then
                -- reposition highlight frames
                CleanUIBattlePetGroupsFrame.scrollFrame.highlightFrame:SetParent(entry);
                CleanUIBattlePetGroupsFrame.scrollFrame.highlightFrame:ClearAllPoints();
                CleanUIBattlePetGroupsFrame.scrollFrame.highlightFrame:SetPoint("TOPLEFT", entry, "TOPLEFT", 0, -5);
                CleanUIBattlePetGroupsFrame.scrollFrame.highlightFrame:SetPoint("BOTTOMRIGHT", entry, "BOTTOMRIGHT", 15, -2);
                CleanUIBattlePetGroupsFrame.scrollFrame.highlightFrame:SetFrameStrata("BACKGROUND");
                CleanUIBattlePetGroupsFrame.scrollFrame.highlightFrame:SetAlpha(0.4);
                CleanUIBattlePetGroupsFrame.scrollFrame.highlightFrame:Show();
            end
        else
            entry:Hide();
        end

        displayedHeight = displayedHeight + buttonHeight;
    end

    HybridScrollFrame_Update(CleanUIBattlePetGroupsFrame.scrollFrame, numEntries * buttonHeight, displayedHeight);
end

function CleanUI_BattlePetGroup_OnEnter(self)
    GameTooltip:SetOwner(self, "ANCHOR_BOTTOMRIGHT", 0, 0);
    GameTooltip:SetText(self.data.name, 1, 1, 1);
    GameTooltip:AddLine(self.data.infoText, 1, 1, 1, true);
    GameTooltip:Show();
end

function CleanUI_BattlePetGroupsPet_OnEnter(self)
    GameTooltip:SetOwner(self, "ANCHOR_BOTTOMRIGHT", 0, 0);
    GameTooltip:SetText(self.speciesName, 1, 1, 1);
    GameTooltip:AddLine(self.sourceText, 1, 1, 1, true);
    if ( not self.tradable ) then
        GameTooltip:AddLine(BATTLE_PET_NOT_TRADABLE, 1, 0.1, 0.1, true);
    end
    if ( self.unique ) then
        GameTooltip:AddLine(ITEM_UNIQUE, NORMAL_FONT_COLOR.r, NORMAL_FONT_COLOR.g, NORMAL_FONT_COLOR.b, true);
    end
    if ( self.description and self.description ~= "" ) then
        GameTooltip:AddLine(" ");
        GameTooltip:AddLine(self.description, nil, nil, nil, true);
    end
    GameTooltip:Show();
end

function CleanUI_BattlePetGroupsPet_OnLeave(self)
    GameTooltip:FadeOut();
end

function CleanUI_BattlePetGroup_ChangeInfoText(self)
    local selectedIndex = self:GetParent().index;
    local data = CleanUIBattlePetGroups.groups[selectedIndex];
    data.infoText = self:GetText();
    CleanUIBattlePetGroups.groups[selectedIndex] = data;

    if (data.infoText and data.infoText ~= "") then
        self:GetParent().editBoxNoInfo:Hide();
    else
        self:GetParent().editBoxNoInfo:Show();
    end
end

function CleanUI_BattlePetGroupsEntry_OnClick(self, button, down)
    selectedIndex = self.index;
    CleanUI_BattlePetGroups_OnUpdate();

    local data = CleanUIBattlePetGroups.groups[selectedIndex];

    if (button == "LeftButton") then
    elseif (button == "RightButton") then
        local dialog = StaticPopup_Show("CUI_BATTLEPET_GROUPS_RENAME_TITLE_POPUP");
        local editbox = _G[dialog:GetName().."EditBox"];
        editbox:SetText(data.name or "");
    end

    CleanUI_BattlePetGroups_ClearFocus();
end

function CleanUI_BattlePetGroupsEntry_OnDoubleClick(self)
    if (not selectedIndex or selectedIndex > #CleanUIBattlePetGroups.groups) then
        return;
    end

    local data = CleanUIBattlePetGroups.groups[selectedIndex];

    for i=1, 3 do
        C_PetJournal.SetPetLoadOutInfo(i, data.pet[i].petGUID);
        C_PetJournal.SetAbility(i, 1, data.pet[i].ability1);
        C_PetJournal.SetAbility(i, 2, data.pet[i].ability2);
        C_PetJournal.SetAbility(i, 3, data.pet[i].ability3);
    end

    PetJournal_UpdateAll();

    CleanUI_BattlePetGroups_SetActiveGroupIndex(selectedIndex);

    CleanUI_BattlePetGroups_ClearFocus();
end


local function compareGroups(group1, group2)
    for i = 1, 3 do
        if (group1.pet[i].petGUID ~= group2.pet[i].petGUID) then
            return false;
        end

        if (group1.pet[i].ability1 ~= group2.pet[i].ability1) then
            return false;
        end

        if (group1.pet[i].ability2 ~= group2.pet[i].ability2) then
            return false;
        end

        if (group1.pet[i].ability3 ~= group2.pet[i].ability3) then
            return false;
        end
    end

    return true;
end


function CleanUI_PetJournal_UpdatePetLoadOut(forceSceneChange)
    if (not CleanUIBattlePetGroups.activeGroupIndex) then
        return;
    end

    -- compare...
    local savedData = CleanUIBattlePetGroups.groups[CleanUIBattlePetGroups.activeGroupIndex];
    local activeData = CleanUI_CollectGroupData(nil);

    if (not compareGroups(savedData, activeData)) then
        CleanUI_BattlePetGroups_SetActiveGroupIndex(nil);
    else
        CleanUI_BattlePetGroups_SetActiveGroupIndex(CleanUIBattlePetGroups.activeGroupIndex);
    end
end

function CleanUI_BattlePetGroups_SetActiveGroupIndex(index)
    CleanUIBattlePetGroups.activeGroupIndex = index;

    if (CleanUIBattlePetGroups.activeGroupIndex) then
        local groupData = CleanUIBattlePetGroups.groups[CleanUIBattlePetGroups.activeGroupIndex];
        PetJournalLoadoutBorderSlotHeaderText:SetText(groupData.name);
    else
        PetJournalLoadoutBorderSlotHeaderText:SetText(BATTLE_PET_SLOTS);
    end
end

function CleanUI_BattlePetGroups_ClearFocus()
    local editBox;
    for i = 1, 20 do
        editBox = _G["CleanUIBattlePetGroupsFrameScrollFrameButton"..i.."EditBox"];
        if (editBox) then
            EditBox_ClearFocus(editBox);
        else
            return;
        end
	end
end

function CleanUI_BattlePetGroups_ResetSelection()
    selectedIndex = nil;
end

function CleanUI_AddPetBattleGroup()
    local dialog = StaticPopup_Show("CUI_BATTLEPET_GROUPS_ADD_TITLE_POPUP");
    local editbox = _G[dialog:GetName().."EditBox"];
    editbox:SetText("");

    if (UnitExists("target")) then
        editbox:SetText(UnitName("target"));
    end
end

function CleanUI_CollectGroupData(name)
    local data = {};
    data.pet = {};

    data.name = name;

    CleanUI_UpdateBattlePetGroupData(data);

    return data;
end

local function compareBattleGroups(group1, group2)
    return group1.name < group2.name;
end

function CleanUI_SortBattleGroups()
    table.sort(CleanUIBattlePetGroups.groups, compareBattleGroups);
end

function CleanUI_AddPetBattleGroupWithName(name)
    local data = CleanUI_CollectGroupData(name);

    tinsert(CleanUIBattlePetGroups.groups, data);
    CleanUI_SortBattleGroups();

    CleanUI_BattlePetGroups_OnUpdate();
end

function CleanUI_RenamePetBattleGroupWithName(name)
    if (not selectedIndex or selectedIndex > #CleanUIBattlePetGroups.groups) then
        return;
    end

    local data = CleanUIBattlePetGroups.groups[selectedIndex];
    data.name = name;
    CleanUI_SortBattleGroups();

    CleanUI_BattlePetGroups_OnUpdate();
end

function CleanUI_SavePetBattleGroup()
    if (not selectedIndex or selectedIndex > #CleanUIBattlePetGroups.groups) then
        return;
    end

    local data = CleanUIBattlePetGroups.groups[selectedIndex];

    CleanUI_UpdateBattlePetGroupData(data);

    CleanUI_BattlePetGroups_OnUpdate();
end

function CleanUI_DeletePetBattleGroup()
    if (not selectedIndex or selectedIndex > #CleanUIBattlePetGroups.groups) then
        return;
    end

    tremove(CleanUIBattlePetGroups.groups, selectedIndex);

    CleanUI_BattlePetGroups_ResetSelection();
    CleanUI_BattlePetGroups_OnUpdate();
end


function CleanUI_UpdateBattlePetGroupData(data)
    local petGUID, ability1, ability2, ability3, locked;
    local petdata;
    for i=1, 3 do
        petGUID, ability1, ability2, ability3, locked = C_PetJournal.GetPetLoadOutInfo(i);
        data.pet[i] = {petGUID = petGUID, ability1 = ability1, ability2 = ability2, ability3 = ability3};
    end
end

------------------------------------------------------------------------------------------------
-- PET BATTLE
------------------------------------------------------------------------------------------------

function CleanUI_Pets_UpdatePetBattleInfo()
    CleanUI_Pets_UpdatePetAllyActions();
    CleanUI_Pets_UpdatePetEnemyActions();

    CleanUI_Pets_UpdateGroupInfo();
end

function CleanUI_Pets_UpdatePetAllyActions()
    CleanUI_Pets_UpdatePetActionButtons(LE_BATTLE_PET_ALLY, -206, -65);
end

function CleanUI_Pets_UpdatePetEnemyActions()
    CleanUI_Pets_UpdatePetActionButtons(LE_BATTLE_PET_ENEMY, 42, -65);
end

function CleanUI_Pets_UpdateGroupInfo()
    local name = "CUI_PetBattle_InfoFrame";
    local infoFrame = _G[name];

    if (not PetBattleFrame or not CleanUIBattlePetGroups.activeGroupIndex) then
        if (infoFrame) then
            infoFrame:Hide()
        end

        return;
    end

    if (not infoFrame) then
        infoFrame = CreateFrame("Frame", name, PetBattleFrame);
        infoFrame:SetSize(280, 80);

        infoFrame:SetPoint("TOPLEFT", UIParent, "TOPLEFT", 20, -20);
        infoFrame:SetPoint("BOTTOMRIGHT", UIParent, "TOPLEFT", 300, -100);

        infoFrame.headline = infoFrame:CreateFontString(infoFrame:GetName().."Title", "ARTWORK");
        infoFrame.headline:SetFontObject("GameFontNormalLarge");
        infoFrame.headline:ClearAllPoints();
        infoFrame.headline:SetPoint("TOPLEFT", infoFrame, "TOPLEFT", 0, 0);

        infoFrame.infoText = infoFrame:CreateFontString(infoFrame:GetName().."InfoText", "ARTWORK");
        infoFrame.infoText:SetFontObject("GameFontNormal");
        infoFrame.infoText:ClearAllPoints();
        infoFrame.infoText:SetPoint("TOPLEFT", infoFrame, "TOPLEFT", 0, -25);
    end

    local data = CleanUIBattlePetGroups.groups[CleanUIBattlePetGroups.activeGroupIndex];

    if (data and data.infoText) then
        infoFrame.headline:SetText("|c333399ff"..data.name.."|r");
        infoFrame.infoText:SetText(data.infoText);

        infoFrame:Show();
    else
        infoFrame:Hide();
    end
end

function CleanUI_Pets_UpdatePetActionButtons(petOwner, x, y)
    if (not PetBattleFrame) then
        return;
    end

    local activeEnemy = C_PetBattles.GetActivePet(petOwner);

    local namePrefix = "Ally";
    if (petOwner == LE_BATTLE_PET_ENEMY) then
        namePrefix = "Enemy";
    end

    local buttonName, button;
    local id, name, icon, maxCooldown, isUsable, currentCooldown;
    for i=1, 3 do

        buttonName = "CUI_PetBattle_"..namePrefix.."Ability"..i;
        button = _G[buttonName];

        if (not button) then
            button = CreateFrame("Button", buttonName, PetBattleFrame, "PetBattleActionButtonTemplate");
            button:SetScale(0.75);
            button:ClearAllPoints();
            button:SetPoint("TOPLEFT", PetBattleFrame, "TOP", x + 56 * (i-1), y);

            button:HookScript("OnEnter", CleanUI_Pets_ShowAbilityTooltip);
            button:HookScript("OnLeave", CleanUI_Pets_HideAbilityTooltip);
        end

        id, name, icon, maxCooldown = C_PetBattles.GetAbilityInfo(petOwner, activeEnemy, i);
        if (id) then
            button.petOwner = petOwner;
            button.index = activeEnemy;
            button.id = i;

            button.Icon:SetTexture(icon);

            button:Show();

            local isUsable, currentCooldown = C_PetBattles.GetAbilityState(petOwner, activeEnemy, i);

            if (currentCooldown < 1) then
                button.CooldownShadow:Hide();
                button.Cooldown:SetText("");
                button.Cooldown:Hide();
            else
                button.CooldownShadow:Show();
                button.Cooldown:SetText(currentCooldown);
                button.Cooldown:Show();
            end
        else
            button:Hide();
        end
    end
end

function CleanUI_Pets_ShowAbilityTooltip(button)
    PetBattleAbilityTooltip_SetAbility(button.petOwner, button.index, button.id);
	PetBattleAbilityTooltip_Show("TOPLEFT", button, "BOTTOMLEFT", 0, -10);
end

function CleanUI_Pets_HideAbilityTooltip()
    PetBattlePrimaryAbilityTooltip:Hide();
end